<?php

/**
* inQuireREST
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

/**
 * @see Zend_Controller_Action
 */
require_once "Zend/Controller/Action.php";
require_once 'Product.php';
require_once 'ProductDetailModel.php';
require_once 'CacheModel.php';
require_once 'WSConstants.php';
require_once 'WSGateway.php';
require_once 'WSFactory.php';
require_once 'WSException.php';
require_once 'Logger.php';
require_once 'InQuireWebUtil.php';

/**
 * @see SmartyDispatcher
 */
require_once "SmartyDispatcher.php";
/**
 * @see DCConfigManagerModel
 */
require_once 'application/models/DCConfigManagerModel.php';
/**
 * @see ConfigFileManagerModel
 */
require_once 'ConfigFileManagerModel.php';
require_once 'Zend/Session/Namespace.php';
/**
 * This calss extends the functionality of Zend_Controller_Action
 * All Common Functionlity of Controllers will reside in this class, 
 * and will override any default functionality of Zend_Controller_Action (if Required)
 *   
 */
class BaseController extends Zend_Controller_Action
{
    //This variable is used to store the start time of any action 
    public $starttime;
    //This variable is used to store the end time of any action	
    public $endtime;
    /**
     * This method overrides the default functionality of Zend_Controller_Action
     * and sets the rootPath and rootDirectory variable from configuration file 
     * and restrict access to logged in users only 
     * @return void  
     */
    public function preDispatch ()
    {
        //Set starttime of the action
        $this->starttime = microtime(true);
        //Request will use session
        new Zend_Session_Namespace();
        //Set variable from configuration File 	
        $config = ConfigFileManagerModel::getInstance();
        //Set rootPath variable
        $this->view->rootPath = $config->getValue('rootPath', 'App');
        //Set rootDirectory variable
        $this->view->rootDirectory = $config->getValue('baseDirectory', 'App');
    }
    /**
     * This method overrides the default functionality of Zend_Controller_Action
     * and calculates the total tome spent to process the selected action 
     * @return void  
     */
    public function postDispatch ()
    {
        //Store the finished time of action
        $this->endtime = microtime(true);
        //Log the total time spent to process that request
        Logger::info("Total Time To Process " . $this->_request->getControllerName() . "/" . $this->_request->getActionName() . "=" . ($this->endtime - $this->starttime) . " Seconds");
    }

    /**
     * This method wil dispatch to the script path where fragment template need to be included and 
     * automatically map it with current script path template directory.
     *
     * @param string $action 
     * @param string $tempalte
     * @return string
     */
    public function renderDynamicTemplates ($action, $template)
    {
        $scripts = $this->view->getScriptPaths();
        $scriptsPath = $scripts[0];
        //$dispatcher = new SmartyDispatcher($scriptsPath);
        $dispatcher = SmartyDispatcher::getInstance($scriptsPath);
        $this->view->setScriptPath($dispatcher->dispatchToTemplatePath($action));
        $fragment = $this->view->render($template . ".tpl");
        $this->view->setScriptPath($dispatcher->dispatchToActionMappedPath());
        return $fragment;
    }
    /**
     * This method renders the header and dislayCategories templates and stores 
     * the HTML response in displayCategories & includeHeader variables of 
     * viewRenders instance that is available on View 
     * 
     * @return void
     */
    public function setHeader ($showSearchBox, $showDC,$populateDCBar=true)
    {
        //Get configFile object
        $config = ConfigFileManagerModel::getInstance();
        //See if DisplayCategory is requested to be shown
        if ($showDC) {
			$dcBar = $this->populateDCBar($config->getValue('catalog', 'inQuire'),$populateDCBar);
			//Check if Web Service Raised error and redirected to error page
			if($dcBar==WSConstants::$ERROR_RESPONSE){
				return WSConstants::$ERROR_RESPONSE;
			}
        	//Populate the Display Bar and set it to displayBar view Object
            $this->view->displayBar = $dcBar;
        }
        //Set whethe we need to show searchBox or not
        $this->view->showSearchBox = $showSearchBox;
        //Include Header
        $this->view->includeHeader = $this->renderDynamicTemplates("includes", "headerLayout");
    }
    /**
     * This method renders the Footer template and stores the HTML response in 
     * includeFooter variable of viewRenders instance that is available on View 
     * 
     * @return void
     */
    public function setFooter ($showSearchBox = true)
    {
        //assign showSearch view object
        $this->view->assign("showSearch", $showSearchBox);
        //Set includeFooter View Object
        $this->view->includeFooter = $this->renderDynamicTemplates("includes", "footerLayout");
    }
    /**
     * This method sets the variable that are required for Home Page rendering  
     * 
     * @return void
     */
    public function setHomePage ()
    {
        //Set Header and show DispayBar and SearchBox
        $header = $this->setHeader(true, true);
		if($header==WSConstants::$ERROR_RESPONSE){
               	return WSConstants::$ERROR_RESPONSE;
        }
                
        //Include the lanfing Template        
        $this->view->landingPage = $this->renderDynamicTemplates("includes", "landingpage");
        //Include Footer and show SearchBox
        $this->setFooter(true);
    }
    /**
     * This method sets the variable that are required for Search Results Page rendering  
     * 
     * @return void
     */
    public function setSearchResultsPage ()
    {
        //Set Header and show DispayBar and SearchBox
        $header = $this->setHeader(true, true);
		if($header==WSConstants::$ERROR_RESPONSE){
               	return WSConstants::$ERROR_RESPONSE;
        }
        //Include the filters Template
        $this->view->filters = $this->renderDynamicTemplates("search", "filters");
        //Include the search Results Template
        $this->view->searchResults = $this->renderDynamicTemplates("search", "srp");
        //Include Footer and show SearchBox
        $this->setFooter(true);
    }
    /**
     * This method sets the variable that are required for Compare Products Page rendering  
     * 
     * @return void
     */
    public function setCompareProductPage ()
    {
        //Set Header and show DispayBar and SearchBox
        $header = $this->setHeader(true, true);
		if($header==WSConstants::$ERROR_RESPONSE){
               	return WSConstants::$ERROR_RESPONSE;
        }
        //Include the Compare Products Template    	
        $this->view->compareProducts = $this->renderDynamicTemplates("compare", "compareProducts");
        //Include Footer and show SearchBox
        $this->setFooter(true);
    }
    /**
     * This is helper method and is used to perform web service request. It makes the web service call and return a php object.
     *
     * @access private
     * @param $parameters, contains the request parameters
     * @param $requestType, which type of request will be created and send to $gateway
     * @return $phpobject, PHP object that is created from the returned XML data
     */
    public function serviceRequest ($parameters, $type)
    {
        $startTime = microtime(true);
        $phpobject = false;
        $gateway = WSGateway::getInstance();
        /**
         * create factory and by using that factory create WSRequest instance
         */
        $wsRequest = WSFactory::createWSRequest(WSConstants::$REST, $type, $parameters);
        if (WSException::$REQUEST_VALIDATION_ERROR !== $wsRequest) {
            /**
             * send the request to gateway
             */
            $wsResponse = $gateway->send(WSConstants::$SIMPLE, $wsRequest);
            if ($wsResponse !== false) {
                /**
                 * return the simple xml object object from the response XML
                 */
                $phpobject = $wsResponse->getSimpleXMLResponse();
            }
        }
        /**
         * return SimpleXMLElement obejct from WSResponse
         */
        $endTime = microtime(true);
        //Log total Time spent in Web Service
        Logger::info("Total Time For WS Processing Type [" . $type . "] Request =" . ($endTime - $startTime) . " Seconds");
        return $phpobject;
    }
    /**
     * This method populates the required data on Search  Result Page
     *
     * @access private
     * @param $phpobject
     * @return $products array of product to be shown
     */
    public function populateSearchResults ($phpobject)
    {
        //Create array to store products	
        $products = array();
        //Iterate over each product
        if ((count($phpobject->products)) > 0) {
	        foreach ($phpobject->products->productSummary as $productSummary) {
	            //Create new Product instance		
	            $product = new Product();
	            //Set Product Id
	            $product->setId($productSummary['id']);
	            //Set Product Short Description
	            $product->setShortDescription(ProductDetailModel::getInstance()->getShortDescription($productSummary));
	            //Set Product Long Description
	            $product->setLongDescription(ProductDetailModel::getInstance()->getLongDescription($productSummary));
	            //Set Product Category
	            $product->setCategory($productSummary->category['name']);
	            //Set Product Manufacurer Number
	            $product->setMfgNumber($productSummary->manufacturer['number']);
	            //Create Skus array
	            $skus = array();
	            //Iterate over each SKU
	            if($productSummary->skus) {
		            foreach ($productSummary->skus->sku as $sku) {
		                if($sku["type"] != 'MFGPARTNUMBER') {
			            	$skus[] = array($sku["type"] , $sku["number"]);
		                }
		            }
	            }
				//Set Skus array in product
	            $product->setSkus($skus);
	            //add product is products array
	            $products[] = $product;
	        }
        }
        //Return Products
        return $products;
    }
    /**
     * This method populates the required data on Search  Result Page
     *
     * @access private
     * @param $phpobject
     * @return $products array of product to be shown
     */
    public function setWSRequestFailed ($reason=null,$detail=null,$suggestion=null)
    {
        //Show Header with DisplayCategory Bar and SearchBox 
        $this->setHeader(true, true,false);
        //Show Footer with SearchBox	 	
        $this->setFooter(true);
        //Log WebService Failure
        Logger::info("WebService Request Failed in Controller:" . $this->getRequest()->getControllerName() . ",Action=>" . $this->_request->getActionName());
        
        if($reason==null){
        	$this->view->reason = "WebService Endpoint";
        }else{
        	$this->view->reason = $reason;
        }

        if($detail==null){
        	$this->view->detail = "Error Occured while calling web service!";
        }else{
        	$this->view->detail = $detail;
        }        

        if($suggestion==null){
        	$this->view->suggestion = "Error Occured while calling web service, Please try again.";
        }else{
        	$this->view->suggestion = $suggestion;
        }        

        //Redirect to Error Page
        $this->_request->setControllerName("error")->setActionName("error");
    }
    /** This method loads the Display Category bar and generates links for that. 
     *  Links are generated by usign Zend_Cache that cache the links generated on 
     *  first user request 
     * @access private
     * @param 
     * @return 
     */
    private function populateDCBar ($catalogName,$populateDCBar=true)
    {
        //Get Config file object
        $config = ConfigFileManagerModel::getInstance();
        //Get Cache object
        $cache = CacheModel::getInstance()->getCache();
        //Get DCBar from cache
        $dcBar = $cache->load('DCBar_' . $catalogName);
		//if $populateDCBar is false don't populate DCBar if not already cached 
        // if dcBar is not available in Cache object than populate that and save in cache
        if ( (!$dcBar) && ($populateDCBar)) {
            //Get all Categories id for default Category			
            $categories = DCConfigManagerModel::getInstance()->getCategoryMap($config->getValue('catalog', 'inQuire'));
            $dcBar = "";
            $index = 0;
            //Get total categories in defaultCatalog
            $categoriesCount = count($categories);
            //Get Cached Categories from Cache
            $cachedCategories = $cache->load('cachedCategories_'.$catalogName);
            //if No cached categories are available than initialze array
            if (! $cachedCategories) {
                $cachedCategories = array();
            }
            //Iterate over each category
            foreach ($categories as $category) {
                //Category already loaded in cache so use that cache value
                if (array_search($category[0], $cachedCategories)) {
                    $dcBar = $cache->save($dcBar, 'incompleteDCBar_'.$catalogName);
                } else {
                    //array to store the categories Ids of the children to Nth level 
                    $catIds = array();
                    //id=0 is reserved for no filtering and showing all products
                    if ($category[0] != 0) {
                        /**
                         * get the child categories information for the given display category id.
                         */
                        $categoryFilterIds = DCConfigManagerModel::getInstance()->getEtilizeCategotyMap($category[0], $catalogName);
                        //get all direct children of current category and store them in an array
                        $etilizeCategotyMapIds = explode(",", $categoryFilterIds);
                        //Iterate over the direct children and retriever all their children to nth level and add them in catIds array
                        foreach ($etilizeCategotyMapIds as $etilizeCategotyMapId) {
							$childCategories = $this->getChildCategories('' . $etilizeCategotyMapId);
							if($childCategories == WSConstants::$ERROR_RESPONSE){
								return WSConstants::$ERROR_RESPONSE;
							}
                        	$catIds = array_merge($catIds, $childCategories);
                            
                        }
                    }
                    //Create the url to be displayed on Display Category Bar
                    $dcBar .= "<li>";
                    $dcBar .= "<a href='" . $config->getValue('rootPath', 'App') . "/search/categorySearch?index=$category[0]&headerName=" . urlencode($category[1]) . "&catIds=" . implode(',', $catIds) . "'>$category[1]</a>";
                    if ($categoriesCount != ($index + 1)) {
                        $dcBar .= "&nbsp;<span class='navDivider'>&nbsp;</span>&nbsp;";
                    }
                    $dcBar .= "</li>";
                    $index ++;
                    //store currently completed category in Cache so if further rocessing fails we continue from this point
                    $cachedCategories[] = $category[0];
                    $cache->save($cachedCategories, 'cachedCategories_'.$catalogName);
                    //cache the incomplete DCBar so we can continue creating DCBar from this point, this 
                    //usually hapend when request timeout occur because of huge number of Web service requests
                    $cache->save($dcBar, 'incompleteDCBar_'.$catalogName);
                }
            }
            // Display Category Bar created successgully so store it in cache 
            $cache->save($dcBar, 'DCBar_' . $catalogName);
            //Remove cachedCategories and dcBar as from cache as we have complete the task for populating the DCBar
            $cache->remove('cachedCategories_'.$catalogName);
            $cache->remove('incompleteDCBar_'. $catalogName);
        }
        //Return cached object of DCBar
        return $dcBar;
    }
    /**
     * Method to Get all the Child Categoris of a category
     *
     * @access private
     * @return $categoriesId, containing Categories IDs as an array
     */
    public function getChildCategories ($categoryId)
    {
        //Set parametes requires to get child categories
        $parameters['categoryId'] = $categoryId;
        
        Logger::info("getChildCategories calling WS");
        //Create array to store the categories ids
        $categoriesId = array();
        //Perform Web Service Request
        $phpobject = $this->serviceRequest($parameters, WSConstants::$CATEGORY_FAMILY);
        //If request processed successfully thatn continue
        if ($phpobject !== false) {
            /*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return WSConstants::$ERROR_RESPONSE;
        	} else{
	            $currentChildren = $phpobject;
	            $categoriesId[] = $categoryId;
	            if (count($currentChildren->children) > 0) {
	                $categoriesId = array_merge($categoriesId, self::getChildIds($currentChildren->children));
	            }
        	}
        }
        return $categoriesId;
    }
    /**
     * Method to Perform recursive collection of child categories
     *
     * @access private
     * @return $categoriesId, containing Categories IDs as an array
     */
    private function getChildIds ($currentChildren)
    {
        //create array to store categoryIds
        $categoriesId = array();
        //Iterate over each category
        foreach ($currentChildren->category as $categoty) {
            //Set category Id
            $categoriesId[] = $categoty['id'];
            // check if we have children of this category than add them recursively  
            if ((count($categoty->children)) > 0) {
                $categoriesId = array_merge($categoriesId, self::getChildIds($categoty->children));
            }
        }
        //Return category ids
        return $categoriesId;
    }
    /**
     * Method to get all Manufaturer and cache them 
     *
     * @access private
     * @return 
     */
    public function getAllManufactures ()
    {
        //Get Config file object
        $config = ConfigFileManagerModel::getInstance();
        //Get object of Cache to retrieve/store links in cache
        $cache = CacheModel::getInstance()->getCache();
        $manufactures = $cache->load('Manufactures_' . $config->getValue('catalog', 'inQuire'));
        
        $parameters['method'] = 'getManufacturers';
        
        // if Manufactureres is not available in Cache object than populate that and save in cache
        if (! $manufactures) {
            Logger::info("getAllManufactures calling WS");
            //Create array to store the manufactures 
            $manufactures = array();
            //Perform Web Service Request
            $phpobject = $this->serviceRequest($parameters, WSConstants::$MANUFACTURER_DIRECTORY);
            //If request processed successfully thatn continue
            if ($phpobject !== false) {
               /*
             	* Check  the returned xml contents for Error message 
			 	*/
        		if(InQuireWebUtil::isErrorResponse($phpobject)){
        			$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            		return WSConstants::$ERROR_RESPONSE;
        		}         
        		   	
                if (count($phpobject) > 0) {
                    foreach ($phpobject->manufacturer as $manufacture) {
                        $manufactures[((string) $manufacture['id'])] = ((string) $manufacture['name']);
                    }
                }
                $cache->save($manufactures, 'Manufactures_' . $config->getValue('catalog', 'inQuire'));
            }
        }
        //Return cached object of DCBar
        return $manufactures;
    }
    /**
     * Method to get all Categories and cache them 
     *
     * @access private
     * @return 
     */
    public function getAllCategories ()
    {
        //Get Config file object
        $config = ConfigFileManagerModel::getInstance();
        //Get object of Cache to retrieve/store links in cache
        $cache = CacheModel::getInstance()->getCache();
        $categories = $cache->load('Categories_' . $config->getValue('catalog', 'inQuire'));
        
        $parameters['method'] = 'getCategories';
        
        // if Categories is not available in Cache object than populate that and save in cache
        if (! $categories) {
            Logger::info("getAllCategories calling WS");
            //Create array to store the categories
            $categories = array();
            //Perform Web Service Request
            $phpobject = $this->serviceRequest($parameters, WSConstants::$CATEGORY_DIRECTORY);
            //If request processed successfully thatn continue
            if ($phpobject !== false) {
               
 			   /*
             	* Check  the returned xml contents for Error message 
			 	*/
        		if(InQuireWebUtil::isErrorResponse($phpobject)){
        			$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            		return WSConstants::$ERROR_RESPONSE;
        		}         
            	
        		if (count($phpobject) > 0) {
                    foreach ($phpobject->category as $category) {
                        $categories[((string) $category['id'])] = ((string) $category['name']);
                    }
                }
                $cache->save($categories, 'Categories_' . $config->getValue('catalog', 'inQuire'));
            }
        }
        //Return cached object of DCBar
        return $categories;
    }
}
		
