<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once 'BaseController.php';
require_once "WSConstants.php";
require_once 'ConfigFileManagerModel.php';
require_once 'Logger.php';
require_once 'InQuireWebUtil.php';

/**
 * This is the default controller, which is required to handle all the root level actions. It typically works 
 * with the urls such as http://inquiredemo.etilize.com/REST/. It forwards this root request to home page.
 * 
 * @todo add other usage
 * 
 * @author Zaid Ansari 
 * @version 1.0
 */
class IndexController extends BaseController
{
    /**
     * The default action - show the home page
     */
    public function indexAction ()
    {
    	//Get Featured Products from cache
    	$featureProducts = $this->getFeatureProducts();

    	//Check if Web Service returned error Response 
		if($featureProducts == WSConstants::$ERROR_RESPONSE){
			return;
		}else //Check if featureProduct object is not false
    	if(!$featureProducts){
	       	//Failed to process Web Service Request Redirect to error page
       		$this->setWSRequestFailed();
    	}else{
			//assign feature products array to view variable.
			$this->view->featureProducts =	$featureProducts;
			//Set Home Page Setting
			$homePage = $this->setHomePage();
    		if($homePage==WSConstants::$ERROR_RESPONSE){
               	return;
        	}			
			//Redirect to Home Page		
	    	$this->_request->setActionName("home");
    	}			
    }

    /**
     * This function populated the feature products for default catalog
     *
     * This Function checks if those feature products are not already cached than
     * it perform Webservice request to fine the feature products, after that it
     * saves the feature products in cache for future calls.
     * If products are already cached than it doesn't make any Web service request 
     * but return the products cached.
     *  
     *  
     */
    private function getFeatureProducts(){

    	//Get Config file object
	 	$config = ConfigFileManagerModel::getInstance();
	 	
	 	//Get object of Cache to retrieve/store links in cache
	 	$cache = CacheModel::getInstance()->getCache();

	 	//Get Default Catalog set in config file
	 	$defaultCatalog = $config->getValue('catalog', 'inQuire');

	 	//Get featuredProducts from Cache
	 	$featureProducts = $cache->load('FeatureProducts_'.$defaultCatalog);
	 	
		// if FeatureProducts for default catalog is not available in cache than load using Web Service and cache it to reduce Web Service calls on each request
		if(!$featureProducts) {
			
			//Get feature Products parse them and saves them in array where key will be the product id and category will be the value
			$featureProductsKey = 'featureProducts_'.$defaultCatalog;
			//featureProductsCSV contains the features products as a string and each prodcut is separated by , sign 
			$featureProductsCSV = $config->getValue($featureProductsKey, 'FeatureProducts');
			
			//This array contain product information, one element contains a string a value like Device:11574226
			$featureProductsArray = split(',',$featureProductsCSV);

			//Create array to store index inproperty file,productid and its category as provided in property file
			$featureProductInfo = array();

			$featureProductIds = "";
			$index=0;
			foreach ($featureProductsArray as $featureProduct){
				 $productInfo = split(':',$featureProduct);
				 $featureProductInfo[(string)$productInfo[1]] = array(''.$index,(string)$productInfo[1],(string)$productInfo[0]);
				 $featureProductIds.=$productInfo[1].",";
				 $index++;
			}
			//Remove extra added, sign 
			$featureProductIds=substr($featureProductIds,0,strlen($featureProductIds)-1);
			
			//Set parameters to perform search
			$parameters['descriptionTypes'] = '1,2';
			//$parameters['includeSkus'] = 'false';
			$parameters['categoryFilters'] = 0;
			$parameters['attributeFilters'] = 0;
			$parameters['manufacturerFilters'] = 0;	
			$parameters['keywordFilter']=	$featureProductIds;						
			$parameters['descriptionTypes'] = '1,2';
			$parameters['pageNo']= '1';
			$parameters['pageSize']='4';

			Logger::info("indexAction Calling WS");
    		//Perform Web Service Request and get the output in $phpobject object
			$phpobject = $this->serviceRequest($parameters, WSConstants::$FEATURED_PRODUCTS);

			//if $phpobject is not false than continue processing else redirect to error page with proper error message
		    if ($phpobject !== false) {

		    	/*
	             * Check  the returned xml contents for Error message 
				 */
	        	if(InQuireWebUtil::isErrorResponse($phpobject)){
	        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
	            	return WSConstants::$ERROR_RESPONSE;
	        	}
	        			    	
		    	//Array to store the feature products
        		$featureProducts = array();
        		
        		//Check if the returned xml contains products element or not
	        	if(count($phpobject->products)>0){
	        		
	        		//Iterate over each productSummary element 	        		
	            	foreach($phpobject->products->productSummary as $product){
						$featureProduct =array();

						//Set Prodcut id in array's 1st index
		            	$featureProduct[0]=(string)$product['id'];

		            	//Get the index of Product in user provided propert file list 
	            		$index = $featureProductInfo[(string)$product['id']][0];
		
		            	//Get the product Type from the same index and set it in 2nd index of array
		            	$featureProduct[1] = $featureProductInfo[(string)$product['id']][2];
						
		            	//set Product Short and Long Description
		            	if(count($product->descriptions)>0){
		            		//Iterate over each descriptions element
			            	foreach( $product->descriptions->description as $description){
			            		if($description['type']=="2"){
		    	        			$featureProduct[2]=(String)$description;	
		        	    		}elseif($description['type']=="1"){
		            				$featureProduct[3]=(String)$description;
		            			}
		            		}
	            		}
	            		//Add one feature product to the specified index in $featureProducts so we can keep them in the same order user provided in properties file
		           		$featureProducts[$index] = $featureProduct;
		           	}
		   		}
		
		       	//Sort Keys so we get the Same order as we have in property file
		    	ksort($featureProducts);

		    	//Save FeatureProducts array in Cache
		    	$cache->save($featureProducts, 'FeatureProducts_'.$defaultCatalog);

       		}else{
       			return false;
       		}    		
		} 
		//Return Feature product that are either created using WS or are from cached.
		return $featureProducts;
	   	   	
    }
    
}
