<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once 'ConfigFileManagerModel.php';
require_once "BaseController.php";
require_once "WSConstants.php";
require_once "ProductDetailModel.php";
require_once 'Logger.php';
require_once 'InQuireWebUtil.php';


/**
 * This controller is used to get the content for any product. It uses REST based architecture to talk to the Etilize web services in order to
 * get these content information. 
 * 
 * Every product has different type of data available which includes, basic specs, detail specs, similar products, option and accessories, suggested
 * products and other. This controller has defined different actions to get this data.
 * 
 * Following url is used to access this controller. This is forwarded to getProductAction that get the basic specs(displayed in overview tab) 
 * suggested products, and category trail.
 * 
 * http://inquiredemo.etilize.com/rest/index.php/product/getProduct?productId=1010042252
 *
 * 
 * 
 * @author Zaid Ansari
 * @version 1.0
 */
class ProductController extends BaseController
{

    /**
     * getproductAction is used to get the information of a single product
     *
     * This function takes one parameter 'productId' and perform WS request to collect all required 
     * information of that product. This fuction than shows the product information on product detail view.
     *
     */
    public function getproductAction ()
    {
    	$config = ConfigFileManagerModel::getInstance();
        //Get product id from request
        $productId = $_GET['productId'];
        //set required parametes to perform Web Service Request
        $parameters['productId'] = $productId;
	$parameters['skuType'] = 'all'; 
        $parameters['resourceType'] = 'all';
        $parameters['descriptionTypes'] = '1,2,3';
        $parameters['categorizeAccessories'] = 'true';
        $parameters['manufacturer'] = 'default';
        $parameters['dataSheet'] = 'basic';
        Logger::info("getproductAction Calling WS");
        //Perform Web Service Request
        
        $phpobject = $this->serviceRequest($parameters, WSConstants::$GET_PRODUCT_DETAIL);
        
        //Web Service request completed successfully
        if ($phpobject !== false) {
        	
            /*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return;
        	}
        	        	
        	
            $_REQUEST["xmldata"] = $phpobject;
            $this->view->xmldata = $phpobject;
            $this->view->product = $phpobject;
            //There are few products whose partial record is entered we cannot show the detail 
            //So redirec that to error page with proper message.e.g product id 1010313353 in na catalog
            if (empty($phpobject->category['id'])) {
                //	Log WebService Failure
                $header = $this->setHeader(true, true);
                if($header==WSConstants::$ERROR_RESPONSE){
                	return;
                }
                $this->setFooter(true);
                Logger::info("Product Detail Not available for product id " . $productId . "in Catalog[" . $config->getValue('catalog', 'inQuire') . "]");
                $this->view->reason = "Product Detail Not Available";
                $this->view->detail = "Product is not available ProductId: " . $productId . ", Catalog:" . $config->getValue('catalog', 'inQuire');
                $this->view->suggestion = "Contact our technical team at integrate@etilize.com with above error description";
                $this->_request->setControllerName("error")->setActionName("error");
                return;
            }
            
            //
   			$parentCategories = $this->getParentCategories($phpobject->category['id']);
   			//check if Web service retuned error response and redirected to Error page
   			if($parentCategories == WSConstants::$ERROR_RESPONSE){
   				return;
   			}   
   			      
            //Get Parent categories of product category and set in parentCategories view object
            $this->view->parentCategories = $parentCategories;

            //set Header and show Display Category and searchBox
            $header = $this->setHeader(true, true);
            if($header == WSConstants::$ERROR_RESPONSE){
            	return;
            }
            
            //Create instance of ProductDetailModel
            $productDetailModel = ProductDetailModel::getInstance();
            //Populate Overview Tab
            
	        $dataSheet = $config->getValue('dataSheet', 'SpeX-Plus Add-Ons');
	        
	        $overview = $productDetailModel->populateOverviewTab($phpobject);
	        
	        if($dataSheet == 'detailed') {
	        	$parameters['dataSheet'] = 'detailed';
	        	$phpobject1 = $this->serviceRequest($parameters, WSConstants::$GET_PRODUCT_DETAIL);
	        	
	        	$overview = $productDetailModel->populateDetailedOverviewTab($phpobject1, $overview);
	        }
	        
	        $this->view->overview = $productDetailModel->formatOverviewTab($overview);
            
            //Technical Specification tab will be populated using Ajax request						
            $this->view->techSpecs = "";
            //Populate Similar Products Tab			
            $this->view->similarProducts = $productDetailModel->formatSimilarProductsTab($phpobject);
            //Populate Option and Accessories Tab			
            $this->view->optAndAccProducts = $productDetailModel->formatOptAndAccTab($phpobject);
            //Set product Id to view object
            $this->view->productId = $productId;
            //These are all possible types of images in Rich Contents
            $possibleRCImages = array("Top" , "Left" , "Front" , "Rear" , "Right");
            //These are Rich Contents types images available for this Product 
            $productRCImages = array();
            if (count($phpobject->resources) > 0) {
                foreach ($phpobject->resources->resource as $resource) {
                    if (in_array('' . $resource['type'], $possibleRCImages, true)) {
                        $productRCImages[] = $resource['type'];
                    }
                }
            }
            //Set RC images for View
            $this->view->productRCImages = $productRCImages;
            $this->createProductAssets($phpobject);
            //Related Products Exists so Render Respective Page
            if (count($phpobject->upsellProducts->productSummary) > 0) {
                $this->view->relatedProducts = ProductDetailModel::getInstance()->populateRelatedProducts($phpobject->upsellProducts->productSummary);
                $this->view->relatedProductsView = $this->renderDynamicTemplates("product", "relatedProducts");
            }
            //set Short and Long Description of Product
            $this->view->shortDesc = ProductDetailModel::getInstance()->getShortDescription($phpobject);
            $this->view->longDesc = ProductDetailModel::getInstance()->getLongDescription($phpobject);
            //Render product info template
            $this->view->productInfo = $this->renderDynamicTemplates("product", "productInfo");
            //Set Footer and show SearchBox
            $this->setFooter(true);
            //redirect to productdetail Template
            $this->_request->setActionName("productdetail");
        } else {
            //Web Service request failed so redirec to error page
            $this->setWSRequestFailed();
        }
    }

    /**
     * gettechnicalAction is used to get the Technical inormation of selected Project
     *
     * This function is invoked when user first selects the Technica Specification Tab.
     * This generates an AJAX Request that retrieves the data from server and shows it to user. 
     *
     */
    public function gettechnicalAction ()
    {
    	$config = ConfigFileManagerModel::getInstance();
    	
        //Its Ajax Request so Set No Renderer
        $this->_helper->viewRenderer->setNoRender();
        //Set productId 
        $productId = $_GET['productId'];
        //Set Parameters required to perform Web service request		
        $parameters['productId'] = $productId;
        $parameters['dataSheet'] = $config->getValue('dataSheet', 'SpeX-Plus Add-Ons');
        
        Logger::info("gettechnicalAction Calling WS");
        //Perform Web service request
        $phpobject = $this->serviceRequest($parameters, WSConstants::$GET_PRODUCT_DETAIL);
        //Web service completed successfully
        if ($phpobject !== false) {
            
			/*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return;
        	}
        	
            //Set response to request
            $_REQUEST["xmldata"] = $phpobject;
            $productDetailModel = ProductDetailModel::getInstance();
            echo $productDetailModel->formatTechnicalSpecsTab($phpobject);
        } else {
            //Web Service Request Failed
            $this->setWSRequestFailed();
        }
    }

    /**
     * getParentCategories is used to get the complete hierarchy of category of current category
     * 
     * This function takes one parameter 'categoryId' and perform WS request to collect all of its parent categories 
     *
     */
    private function getParentCategories ($categoryId)
    {
    	$config = ConfigFileManagerModel::getInstance();
    	
        //Set parameters required to perform Web Service Request
        $parameters['categoryId'] = $categoryId;
        $parameters['childLevels'] = '0';
        Logger::info("getParentCategories Calling WS");
        $parentCategories = array();
        $formattedParentCategories = "";
        //Perform Web Service
        $phpobject = $this->serviceRequest($parameters, WSConstants::$CATEGORY_FAMILY);
        //Web Service completed successfully
        if ($phpobject !== false) {
        	
            /*
             * Check  the returned xml contents for Error message 
		 	 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return WSConstants::$ERROR_RESPONSE;
        	} 
        		        	
            //Set current Category to the root Element
            $currentCategory = $phpobject;
            //store current Category id and name in array        	
            $parentCategories[] = array($currentCategory['id'] , $currentCategory['name']);
            //Perform loop till you reach the top level of category        	
            while ($currentCategory->parent['id'] != 1) {
                $currentCategory = $currentCategory->parent;
                $parentCategories[] = array($currentCategory['id'] , $currentCategory['name']);
            }
            //Reverse the array to siplay Top Hierarchy first
            $parentCategories = array_reverse($parentCategories);
            //Format and Create Links 
            $formattedParentCategories .= "<div id=\"categories\" class=\"a12BlueB\">";
            foreach ($parentCategories as $key => $value) {
                $formattedParentCategories .= "<a href=\"".$config->getValue('rootPath', 'App')."/search/categorySearch?selectedCategoryId=$value[0]"."&headerName=" . urlencode($value[1]) . "\" class=\"a12BlueB\">";
                $formattedParentCategories .= $value[1];
                $formattedParentCategories .= "</a>";
                if ($key < (count($parentCategories) - 1)) {
                    $formattedParentCategories .= " / ";
                }
            }
            $formattedParentCategories .= "</div>";
        }
        return $formattedParentCategories;
    }

    private function createProductAssets ($phpobject)
    {
        if (count($phpobject->resources) > 0) {
            foreach ($phpobject->resources->resource as $resource) {
                switch ($resource['type']) {
                    case "300":
                        $this->view->is300Published = "true";
                        break;
                    case "Thumbnail":
                        $this->view->isThumbnailPublished = "true";
                        break;
                    case "1300":
                        $this->view->is1300Published = "true";
                        break;
                    case "Original":
                        $this->view->isOriginalPublished = "true";
                        break;
                    case "Tour":
                        $this->view->isTourPublished = "true";
                        break;
                    case "Life-Style":
                        $this->view->isLSPublished = "true";
                        break;
                    case "650":
                        $this->view->is650Published = "true";
                        break;
                    case "Jack-Pack":
                        $this->view->isJPPublished = "true";
                        break;
                    case "User-Manual":
                        $this->view->isUMPublished = "true";
                        break;
                    case "Manufacturer-Brochure":
                        $this->view->isMBPublished = "true";
                        break;
                }
            }
        }
    }
}
		
		
