<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once "Product.php";
	require_once 'ConfigFileManagerModel.php';
	 
	class ProductDetailModel {
		/**
		* static property to hold singleton instance
		*/
		private static $INSTANCE = false;
		
		
		/**
		* factory method to return the singleton instance
		* @return ProductDetailModel
		*/	
		static public function getInstance() {
			if (!ProductDetailModel::$INSTANCE) {
				ProductDetailModel::$INSTANCE = new ProductDetailModel();
				//self::$config = new Zend_Config_Ini('application/inquire.web.ini','authentication');
			}
			return ProductDetailModel::$INSTANCE;
		}

		/**
		* constructor
		* private so only getInstance() method can instantiate
		* @return void
		*/	
		private function __construct() {
			
		}	


		
		public function populateOverviewTab($phpobject){
				$overview = array();
				$attributes = array();
				$mktInformation = "";
				if(count($phpobject->datasheet)>0){		
					foreach( $phpobject->datasheet->attributeGroup as $attributeGroup){
						if($attributeGroup["id"]==35){
							//get Marketing Information
							foreach( $attributeGroup->attribute as $attribute){
		            			if($attribute["name"]=="Marketing Information"){
		            				$mktInformation = $attribute;
		            				break;
		            			}
		            		}
						}else{
							foreach( $attributeGroup->attribute as $attribute){
		            			$attributeKey = ''.$attribute["name"];
		            			$attributes[$attributeKey] = $attribute;
		            		}
						}
					}
				}
				if(count($attributes) > 0) {
					$overview[] = array("Main Features"=>$attributes);
				} else {
					$overview[] = array(""=>"No Data Available.");
				}
				
				if($mktInformation != ""){
					$overview[] = array("Marketing Information"=>$mktInformation);
				}
				return $overview;
	    }
	    
	    public function populateDetailedOverviewTab($phpobject, $overview){
	    	
	    	if(count($phpobject->datasheet)>0){
				foreach( $phpobject->datasheet->attributeGroup as $attributeGroup){
					if($attributeGroup["id"]==57){
						//get Marketing Information
						foreach( $attributeGroup->attribute as $attribute){
	            			if($attribute["name"]=="Package Contents"){
	            				$overview[]["Package Contents"] = array('Contents'=>$attribute);
	            				break;
	            			}
	            		}
	            	}
				}
			}
			return $overview;
	    }
	    
	    public function formatOverviewTab($overview){
			$formattedOverview = "";
	    	foreach( $overview as $section){
				foreach( $section as $heading=>$attribute){
						$formattedOverview.="<div style='font-size:13px;' class=\"padT10 a12BlackB\">";
						$formattedOverview.=$heading;
						$formattedOverview.="</div>";
						$formattedOverview.="<div class='dataSep'> <div style='line-height:18px;' class='padT10 a12Black'> ";
						
						if(is_array($attribute)){
							foreach( $attribute as $attributeKey=>$attributeValue){
								$formattedOverview.="<li class='gBullet'><span style='line-height:18px;' class='a12Black'>";
								$formattedOverview.=$attributeKey.": ".$this->formatOverviewValues($attributeValue);
								$formattedOverview.="</span></li>";
							}
						}else{
								$formattedOverview.="".$attribute;
						}
						$formattedOverview.="</div></div>";
				}
	    	}
	    	return $formattedOverview;
	    }		
	    
		
		private function populateTechnicalSpecsTab($phpobject){
				$techSpecs = array();
				
	            foreach( $phpobject->datasheet->attributeGroup as $attributeGroup){
	            	$techSpecsAttributes =array();	
	            	
	            	foreach( $attributeGroup->attribute as $attribute){
	            		$attributeKey = ''.$attribute["name"];
	            		$techSpecsAttributes[$attributeKey] = $attribute;
	            	}
	            	
	            	$attributeGroupkey=''.$attributeGroup["name"];
	            	$techSpecs[$attributeGroupkey]=$techSpecsAttributes;
				}
				return $techSpecs;
	    }
	    
	    public function formatTechnicalSpecsTab($phpobject){
			$techSpecs	= $this->populateTechnicalSpecsTab($phpobject);
	    	$formattedTechSpecs = "";
			foreach( $techSpecs as $k=>$v){
					$formattedTechSpecs.="<div style='font-size:13px;' class=\"padT10 a12BlackB\">";
					$formattedTechSpecs.=$k;
					$formattedTechSpecs.="</div>";
					$formattedTechSpecs.="<div class='dataSep'> <div style='line-height:18px;' class='padT10 a12Black'>";
	
					$formattedTechSpecs.="<table class=\"product\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">";
					$formattedTechSpecs.="<tbody>";
					
					foreach( $v as $kk=>$vv){
						$formattedTechSpecs.="<tr>";
						$formattedTechSpecs.="<td align=\"right\" valign=\"top\" width=\"180\">";
						$formattedTechSpecs.="<span class=\"a12Black\">";
						$formattedTechSpecs.=$kk;
						$formattedTechSpecs.=": </span>";
						$formattedTechSpecs.="</td>";
						$formattedTechSpecs.="<td class='padL10' style='list-style:inside;' valign=\"top\">";
						$formattedTechSpecs.="<span class=\"a12Black\">";
						$formattedTechSpecs.=$vv;
						$formattedTechSpecs.="</span>";
						$formattedTechSpecs.="</td>";
						$formattedTechSpecs.="</tr>";
					}
					$formattedTechSpecs.="</tbody></table> </div></div>";
			}
	    	return $formattedTechSpecs;
	    }

	 		
	    
	    private function populateSimilarProductsTab($phpobject){
				$similarProducts = array();
				if(count($phpobject->similarProducts)>0){
		            foreach( $phpobject->similarProducts->productSummary as $similarProduct){
		            	$product = new Product();
						$product->setId($similarProduct['id']);
						$product->setShortDescription($this->getShortDescription($similarProduct));
						$product->setLongDescription($this->getLongDescription($similarProduct));
		            	$product->setMfgNumber($similarProduct->manufacturer['number']);
	
		            	$skus = array();
		            	if(count($similarProduct->skus)>0){
							foreach ($similarProduct->skus->sku as $sku){
								if (($sku["type"] != 'UNSPSC') && ($sku["type"] != 'UPC') && ($sku["type"] != 'United') && ($sku["type"] != 'MFGPARTNUMBER') && ($sku["type"] != 'SP Richards Old')){
									$skus[] = array($sku["type"],$sku["number"]);
			    				} 
							}
		            	}
						$product->setSkus($skus);	
		           	
		   	            $similarProducts[] = $product;
					}
				}
				return $similarProducts;
	    }
	    
	    public function formatSimilarProductsTab($phpobject){
	    	$similarProducts	= $this->populateSimilarProductsTab($phpobject);
			$config = ConfigFileManagerModel::getInstance();
			$formattedSimilarProducts="";

	    	foreach($similarProducts as $similarProduct){
				$formattedSimilarProducts.="<div id='productDetail' class='fltL padT5'>";
					$formattedSimilarProducts.="<div class='productResultImg'>";
						$formattedSimilarProducts.="<a href=".$config->getValue('rootPath', 'App')."/product/getProduct?productId=".$similarProduct->getId()."><img src='http://content.etilize.com/images/75/".$similarProduct->getId().".jpg' border=\"0\" /></a>";				
					$formattedSimilarProducts.="</div>";

					$formattedSimilarProducts.="<div class='productResultT' style='margin-top:5px;margin-left:15px;width:470px;'>";
						
						$formattedSimilarProducts.="<a class='a12BlueB' href=".$config->getValue('rootPath', 'App')."/product/getProduct?productId=".$similarProduct->getId()."><div class='a12BlueB'>";
							$formattedSimilarProducts.=$similarProduct->getShortDescription();
						$formattedSimilarProducts.="</div></a>";

						$formattedSimilarProducts.="<div class='a11Black'>";
							$formattedSimilarProducts.=$similarProduct->getLongDescription();
						$formattedSimilarProducts.="</div>";

						$formattedSimilarProducts.="<div class='a11DGray'>";
							foreach( $similarProduct->getSkus() as $sku){
								$formattedSimilarProducts.=$sku[0].": ".$sku[1].", ";
							}
							//Remove , and space character if any sku is present
							if(count($similarProduct->getSkus())>0){
								$formattedSimilarProducts = substr($formattedSimilarProducts,0,count($formattedSimilarProducts)-3);
							}	
						$formattedSimilarProducts.="</div>";
						
						$formattedSimilarProducts.="<div class='a11DGray'>";
							$formattedSimilarProducts.="Mfg Number: ".$similarProduct->getMfgNumber();
						$formattedSimilarProducts.="</div>";
					
					$formattedSimilarProducts.="</div>";
					
				$formattedSimilarProducts.="</div>";				   				
			}
			return $formattedSimilarProducts;
    	    		
	    }
	
	    
	 
	    private function populateOptAndAccTab($phpobject){
				//this willcontain the categories and their respective products
	    		$categories = array();
	    		if(count($phpobject->categorizedAccessories)>0){
					foreach( $phpobject->categorizedAccessories->accessoryCategory as $accessoryCategory){
			    		//this will contain theoption and accessories for one category
	    				$optionAccessories = array();
						$categoryId   = $accessoryCategory->category["parId"];
						$categoryName = $accessoryCategory->category["name"];
						if(count($accessoryCategory->accessories)>0){
							foreach( $accessoryCategory->accessories->productSummary as $accessoryProduct){
			            		$product = new Product();
								$product->setId($accessoryProduct['id']);
								$product->setShortDescription($this->getShortDescription($accessoryProduct));
								$product->setLongDescription($this->getLongDescription($accessoryProduct));
			            		$product->setMfgNumber($accessoryProduct->manufacturer['number']);
			            		
			    				$skus = array();
			    				if(count($accessoryProduct->skus)>0){
									foreach ($accessoryProduct->skus->sku as $sku){
										if (($sku["type"] != 'UNSPSC') && ($sku["type"] != 'UPC') && ($sku["type"] != 'United') && ($sku["type"] != 'MFGPARTNUMBER') && ($sku["type"] != 'SP Richards Old')){
											$skus[] = array($sku["type"],$sku["number"]);
										} 
									}
			    				}
								$product->setSkus($skus);	
			   	            	$optionAccessories[] = $product;
							}
						}
						$categories[]=array($categoryId,$categoryName,$optionAccessories);				
					}
	    		}
							
				return $categories;
	    }
	    
	    public function formatOptAndAccTab($phpobject){
	    	$categories	= $this->populateOptAndAccTab($phpobject);
			$config = ConfigFileManagerModel::getInstance();
			$formattedOptAndAccProducts="";
	
			foreach($categories as $category){
				//$formattedOptAndAccProducts.=$category[1]."<div class='dataSep'> 111</div>";
				//$formattedOptAndAccProducts.="<div class='dataSep'> </div>";
				$formattedOptAndAccProducts.="<div style='padding-top:10px;margin-bottom:5px;border-bottom:2px solid #ccc;width:95%;font-size:12px;clear:both;' class='fltL a12BlueB'> $category[1] </div>";
				
		    	foreach($category[2] as $optionAccessoryProduct){
		    							
					$formattedOptAndAccProducts.="<div id='productDetail' class='padTB5 fltL'>";
						
						$formattedOptAndAccProducts.="<div class='productResultImg fltL'>";
							$formattedOptAndAccProducts.="<a href=".$config->getValue('rootPath', 'App')."/product/getProduct?productId=".$optionAccessoryProduct->getId()."><img src='http://content.etilize.com/images/75/".$optionAccessoryProduct->getId().".jpg' border=\"0\" /></a>";
						$formattedOptAndAccProducts.="</div>";

						$formattedOptAndAccProducts.="<div class='productResultT fltL' style='margin-top:5px;margin-left:15px;width:470px;'>";
						
						$formattedOptAndAccProducts.="<a 'a12BlueB' href=".$config->getValue('rootPath', 'App')."/product/getProduct?productId=".$optionAccessoryProduct->getId()."><div class='a12BlueB'>";
							$formattedOptAndAccProducts.=$optionAccessoryProduct->getShortDescription();								
						$formattedOptAndAccProducts.="</div></a>";
	
						$formattedOptAndAccProducts.="<div class='a11Black'>";
							$formattedOptAndAccProducts.=$optionAccessoryProduct->getLongDescription();
						$formattedOptAndAccProducts.="</div>";
						
						$formattedOptAndAccProducts.="<div class='a11DGray'>";
							foreach( $optionAccessoryProduct->getSkus() as $sku){
								$formattedOptAndAccProducts.=$sku[0].": ".$sku[1].", ";
							}
							//Remove , and space character if any sku is present
							if(count($optionAccessoryProduct->getSkus())>0){
								$formattedOptAndAccProducts = substr($formattedOptAndAccProducts,0,count($formattedOptAndAccProducts)-3);
							}		
							$formattedOptAndAccProducts.="</div>";
						
							$formattedOptAndAccProducts.="<div class='a11DGray'>";
								$formattedOptAndAccProducts.="Mfg Number: ".$optionAccessoryProduct->getMfgNumber();
							$formattedOptAndAccProducts.="</div>";

						$formattedOptAndAccProducts.="</div>";
					
					$formattedOptAndAccProducts.="</div>";
							
											
				}
				//$formattedOptAndAccProducts.="<div class='dataSep'> 222</div>";
				//$formattedOptAndAccProducts.="<div style='font-size:13px;clear:both;' class='fltL a12BlueB'> $category[1] </div>";
				
			}
	    	return $formattedOptAndAccProducts;
	    	    		
	    }   
	
	    
	    
		public static function getShortDescription($product){
			$shortDescription = "";
			if(count($product->descriptions)>0){
				//We will display the desctiption that has attribute pt=1 if pt=2 is missing
				$alternateShort;
				foreach ($product->descriptions->description as $description){
					if ($description != "") {
						if ($description["type"] == "2"){
							$shortDescription= $description;
							break;
						} elseif($description["type"] == "1") {
							$alternateShort=$description;
						}
					} else {
						$shortDescription = 'No Description Available';
					}
				}
				if($shortDescription==""){
					$shortDescription = $alternateShort;
				}
			}
			return $shortDescription;
		}	

		public static function getLongDescription($product){
			$longDescription = "";
			if(count($product->descriptions)>0){
				foreach ($product->descriptions->description as $description){
					if ($description["type"] == "3"){
						$longDescription= $description;
						break;
					}
				}
			}
			return $longDescription;
		}	
		
		public static function populateRelatedProducts($upsellProducts){
			$relatedProducts = array();
			foreach ($upsellProducts as $upsellProduct){
				$relatedProduct = array();
				$relatedProduct[0]=$upsellProduct["id"];
				$relatedProduct[1]=ProductDetailModel::getShortDescription($upsellProduct);
				$relatedProduct[2]=ProductDetailModel::getLongDescription($upsellProduct);
				
				$relatedProducts[] = $relatedProduct;
			}

			return $relatedProducts;
		}
		
		//Special Formatting For OVerview Values
		private function formatOverviewValues($value){
			$value=str_ireplace("<p>","",$value);
			$value=str_ireplace("</p>","",$value);
			$value=str_ireplace("<b>","",$value);
			$value=str_ireplace("</b>","",$value);
			
			$value=str_ireplace("<br>","<li>",$value);
			$value=str_ireplace("</br>","",$value);
			$value=str_ireplace("<i>","",$value);
			$value=str_ireplace("</i>","",$value);
			
			$value=str_ireplace("<LI>","<li>",$value);
			$value=str_ireplace("<UL>","<ul>",$value);	
			$value=str_ireplace("</li>","",$value);									

	
			$value=str_ireplace("</ul>","",$value);
			$value=str_ireplace("\n","",$value);									

	
			$value=str_ireplace("\r","",$value);

       		/* now after it's cleaned, let's format how we want */
			$value=str_ireplace("</ul>","",$value);
			$value = $this->replaceFirstOccurance("<li>","",$value);
			$value=str_ireplace("<li>",", ",$value);								

		
			$value=str_ireplace("<ul>","",$value);
			return $value;
		}

		//Function For replacing only First Instance of one string in another string.		
		function replaceFirstOccurance ( $ReplaceThis, $WithThis, $InThisString ){
			$strPos = strpos( $InThisString,$ReplaceThis);
	    	if ($strPos !== false) {
        		$InThisString = substr ( $InThisString, 0, $strPos ) . $WithThis . substr ( $InThisString, $strPos + strlen ($ReplaceThis ) );			
			}
	    	return $InThisString;
		}
	
		
	}