<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once 'ConfigFileManagerModel.php';
require_once 'Logger.php';

/**
 * Code that failed or throw some exception and need to be re-executed that code for some type of exceptions 
 * will be implemented with this abstract class.
 *
 * We will implement this abstract class in any class that use retry code and override execute 
 * method that contains code that is to be re-executed.
 * 
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 * 
 */
abstract class RetryCode
{

    /**
     * Execute method will be implemened by class that's inherit RetryCode and add retry logic.
     *
     * @param array $arguments
     */
    abstract protected function execute ($arguments);

    /**
     * Method that runs the retry logic. In uses following configuration parameters in the retry logic.
     * 
     * <ul>
     * <li> maxRetries </li>
     * <li> retryEnabled </li>
     * <li> retryWaitTime </li>
     * </ul>
     *
     * @param RetryCode $retryCode
     * @param array $arguments
     * @return Object
     */
    public function run (RetryCode $retryCode, $arguments)
    {
        $returnObj = false;
        $config = ConfigFileManagerModel::getInstance();
        $max_retries = $config->getValue('maxRetries', 'SLService');
        $retry_wait_time = $config->getValue('retryWaitTime', 'SLService');
        $retry_count = 0;
        /**
         * code that retries until got the response or counts exceed the given maximum retries.
         */
        while (! $returnObj && $retry_count < $max_retries) {
            try {
                $returnObj = $retryCode->execute($arguments);
                if ($returnObj != false) {
                    break;
                } else {
                    $retry_count ++;
                }
                Logger::info("Retrying after: $retry_wait_time . ( $retry_count ) \n");
            } catch (Exception $e) {
                Logger::error("Exception cought by RetryCode. re-trying after: $retry_wait_time . ( $retry_count ) Retry \n" . $e->getMessage() . "\nDetail=>" . $e->getTraceAsString());
                sleep($retry_wait_time);
                $retry_count ++;
            }
        }
        if ($retry_count == $max_retries) {
            Logger::error("Request Failed -> Retries " . $retry_count . " times but didn't get result for service. might service is down. check logs for details.");
        }
        return $returnObj;
    }
}
?>