<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/


/**
 * SearchFilterListModel is used to format lazy filters list retrieve through ajax call.
 *
 * SearchFilterListModel is used to create following filters list.
 *
 * <li>Categories List</li>
 * <li>Manufacturers List</li>
 * <li>Attributes List</li>
 *
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0 *
 * 
 * @see SearchController
 * @see SearchTrail
 */
class SearchFilterListModel
{
    private $searchFilters = null;
    private $path = null;
    private $trailString = null;

    public function __construct($searchFilters, $trailString)
    {
        $config = ConfigFileManagerModel::getInstance();
        $this->path = $config->getValue('rootPath', 'App');
        $this->searchFilters = $searchFilters;
        $this->trailString = $trailString;        
    }

    private function createCategoryFiltersList ()
    {
        $counter = 0;
        $formattedFiltersList = "";
        if (count($this->searchFilters->categoryFilters) > 0) {
            foreach ($this->searchFilters->categoryFilters->cf as $category) {
                if ($counter >= 5) {
                    $formattedFiltersList .= "<a class='a11Blue' href='$this->path/filter/category?id=" . $category['id'] . "&trail=" . urlencode($this->trailString) . "'>";
                    $formattedFiltersList .= "<div class='a11Blue'> ";
                    $formattedFiltersList .= $category['name'] . "(" . $category['count'] . ")";
                    $formattedFiltersList .= " </div>";
                    $formattedFiltersList .= "</a>";
                }
                $counter ++;
            }
        }
        return $formattedFiltersList;
    }

    private function createManufacturerFiltersList ()
    {
        $counter = 0;
        $formattedFiltersList = "";
        if (count($this->searchFilters->manufacturerFilters) > 0) {        	
            foreach ($this->searchFilters->manufacturerFilters->mf as $manufacturer) {
                if ($counter >= 5) {
                    $formattedFiltersList .= "<a class='a11Blue' href='$this->path/filter/manufacturer?id=" . $manufacturer['id'] . "&trail=" . urlencode($this->trailString) . "'>";
                    $formattedFiltersList .= "<div class='a11Blue'> ";
                    $formattedFiltersList .= $manufacturer['name'] . "(" . $manufacturer['count'] . ")";
                    $formattedFiltersList .= " </div>";
                    $formattedFiltersList .= "</a>";
                }
                $counter ++;
            }
        }
        return $formattedFiltersList;
    }

    private function createAttributeFiltersList ()
    {
        $counter = 0;
        $formattedFiltersList = "";
        if (count($this->searchFilters->attributeFilters) > 0) {
            foreach ($this->searchFilters->attributeFilters->attribute as $attribute) {
                $abs = true;
                if ($counter >= 5) {
                    $formattedFiltersList .= '<div id="filterProperty">';
                    $formattedFiltersList .= '<div class="a11Black" style="padding:2px 0px;"> <span style="border-bottom:1px dotted">' . $attribute['name'] . '</span> </div>';
                    $formattedFiltersList .= '<form name="attributes_' . $attribute['id'] . '" action="' . $this->path . '/filter/attribute" method="get">';
                    $formattedFiltersList .= '<input type="hidden" name="trail" value="' . $this->trailString . '">';
                    $formattedFiltersList .= '<input type="hidden" name="operator" id="operator" value="0">';
                    foreach ($attribute->value as $attfilter) {
                        if ($attribute['type'] == "numeric") {
                            if ($abs == true) {
                                $abs = false;
                                $formattedFiltersList .= '<select class="a11Black filterSelect" id="selectBox" name="absoluteOperator" style="margin-right:4px;width:50px;" onchange="$(operator).value = this.options[this.selectedIndex].value">';
                                $formattedFiltersList .= '<option value="0">=</option>';
                                $formattedFiltersList .= '<option value="1">&gt;=</option>';
                                $formattedFiltersList .= '<option value="2">&lt;=</option>';
                                $formattedFiltersList .= '</select>';
                            }
                        }
                    }
                    if ($attribute['type'] == "string") {
                        $formattedFiltersList .= '<select name="attributefilter" class="a11Black filterSelect" style="width:208px;" onchange="attributes_' . $attribute['id'] . '.submit()">';
                    } else {
                        $formattedFiltersList .= '<select name="attributefilter" class="a11Black filterSelect" style="width:154px;" onchange="attributes_' . $attribute['id'] . '.submit()">';
                    }
                    $formattedFiltersList .= '<option value=""> All (' . $attribute['count'] . ')</option>';
                    $formattedFiltersList .= '<option value="">---------------</option>';
                    $value = "";
                    foreach ($attribute->value as $attfilter) {
                    	if ($attribute['type'] == "string") {
                            $value = $attribute['id'] . "|";
                            $value .= $attfilter['id'];
                            $value .= "!";
                            $displayValue = str_replace("+", "%2B", $attfilter['display']);
                            $displayValue = str_replace("'", "~", $displayValue);
                            $value .= $displayValue;
                        } else {
                            $value = $attribute['id'] . "|";
                            $value .= $attfilter['id'];
                            $value .= "!";
                            $displayValue = str_replace('"', '`', $attfilter['display']);
                            $value .= $displayValue;
                        }
                        $formattedFiltersList .= '<option value="' . $value . '">' . $attfilter['display'] . '(' . $attfilter['count'] . ')' . ' </option>';
                    }
                    $formattedFiltersList .= '</select></form></div>';
                }
                $counter ++;
            }
        }
        return $formattedFiltersList;
    }

    public function formatFiltersList ($type)
    {
        $filtersList = "";
        if ($type == "category") {
            $filtersList = $this->createCategoryFiltersList();
        } else if ($type == "manufacturer") {
            $filtersList = $this->createManufacturerFiltersList();
        } else {
            $filtersList = $this->createAttributeFiltersList();
        }
        return $filtersList;
    }
}
?>