<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

/**
 * SmartyDispatcher class is used resolve the path for template files those are need 
 * to be included in the current template file. it controls dynamic mapping of template 
 * directories and dispatch request request to appropriate template for rendering.
 *
 * Note : If we need to include any other fragment template file inside an exisitng 
 * template then we need to use this class and provide the path of the including template 
 * file. The including file script is rendered and assign in the view variable for use in 
 * the in the template file where required. All fragment that are needed to be include in 
 * the target template file use the above class to dynamically map the target script path 
 * and forward to the actually when the required fragments are rendered in view variables.
 * 
 *
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 * @see SmartyView
 * @see SmartyViewLoader
 */
class SmartyDispatcher
{
    /**
     * static property to hold singleton instance
     */
    private static $INSTANCE = false;
    private $scriptsRoot = false;

    /**
     * factory method to return the singleton instance
     * @return SmartyDispatcher
     */
    static public function getInstance ($scriptsRoot)
    {
        if (! SmartyDispatcher::$INSTANCE) {
            SmartyDispatcher::$INSTANCE = new SmartyDispatcher($scriptsRoot);
        }
        return SmartyDispatcher::$INSTANCE;
    }

    /**
     * constructor
     * private so only getInstance() method can instantiate
     * @return void
     */
    private function __construct ($scriptsRoot)
    {
        $this->scriptsRoot = $scriptsRoot;
    }

    public function dispatch ()
    {
    }

    /**
     * Dispatch the script path to inlcuding template file dir. 
     *
     * @param strng $dispatchPath
     * @return string
     */
    public function dispatchToTemplatePath ($dispatchPath)
    {
        $scriptPath = $this->scriptsRoot . $dispatchPath;
        return $scriptPath;
    }

    /**
     * Dispatch the path where target template file exist.
     *
     * @return string
     */
    public function dispatchToActionMappedPath ()
    {
        $scriptPath = $this->scriptsRoot;
        return $scriptPath;
    }
}
?>