<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once 'Smarty/Smarty.class.php';

/**
 * SmartyView is the Wrapper class around Zend_View and provides entry point for Smarty 
 * related activities.
 * 
 * SmartyView just provide a wrapper object around Zend_View_Abstract class and support 
 * all the capability of calling Smarty functions, modifiers, templates and all available 
 * features provided by Smarty.
 * 
 * This class provides both facility to use it like Zend_View assign variables in Controller 
 * as specifies in Zend Documentation and use it in template file although you have the 
 * functionality in smarty template to get all variable assign through zend or 
 * application objects (request, session, global e.t.c). 
 * 
 * You can also get Smarty engine from this class object by calling it method getEngine().
 * 
 * SmartyView forward request to target template file. It will map the template directory 
 * according to the Zend MVC view directory specification.
 *
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 * @see SmartyViewLoader
 * @see Zend_View_Abstract
 * @see index.php (Front Controller)
 */
class SmartyView extends Zend_View_Abstract
{
    protected $_smarty;

    public function __construct ($config = array())
    {
        /**
         * Create the smarty object that can be use by Zend Viw in the application.
         */
        $this->_smarty = new Smarty();        
        if (! isset($config['compileDir']))
            throw new Exception('compileDir is not set for ' . get_class($this)); else
            $this->_smarty->compile_dir = $config['compileDir'];
        if (isset($config['configDir']))
            $this->_smarty->config_dir = $config['configDir'];
        if (isset($config['pluginDir']))
            $this->_smarty->plugin_dir[] = $config['pluginDir'];
        $this->registerFunctions();
        parent::__construct($config);
        
        /**
         * Set the template to cache to SmartyEngine
         */
        /*
        $this->_smarty->caching = 2;
        $this->_smarty->compile_check = TRUE;
        // Temprorary add this value first to check it performance them perform some cleanup.
        $this->_smarty->cache_lifetime = 43200;
		*/
    }

    /**
     * Get the Smarty Engine for smarty related activities.
     *
     * @return Smarty obejct
     */
    public function getEngine ()
    {
        return $this->_smarty;
    }

    /**
     * Sets the key value inside smarty assign variables those are used in templates where
     * required.
     *
     * @param string $key
     * @param string $val
     */
    public function __set ($key, $val)
    {
        $this->_smarty->assign($key, $val);
    }

    /**
     * Enter description here...
     *
     * @param unknown_type $key
     * @return unknown
     */
    public function __isset ($key)
    {
        $var = $this->_smarty->get_template_vars($key);
        if ($var)
            return true;
        return false;
    }

    /**
     * Remove specified key from assigned variable
     *
     * @param string $key
     */
    public function __unset ($key)
    {
        $this->_smarty->clear_assign($key);
    }

    /**
     * Assign key or value
     *
     * @param string $spec
     * @param string $value
     */
    public function assign ($spec, $value = null)
    {
        if ($value === null)
            $this->_smarty->assign($spec); else
            $this->_smarty->assign($spec, $value);
    }

    /**
     * Clear all smarty assgined variables
     */
    public function clearVars ()
    {
        $this->_smarty->clear_all_assign();
    }

    /**
     * Call parent run method
     */
    protected function _run ()
    {
        $this->strictVars(true);
        $this->_smarty->assign_by_ref('this', $this);
        $templateDirs = $this->getScriptPaths();
        $file = substr(func_get_arg(0), strlen($templateDirs[0]));
        $this->_smarty->template_dir = $templateDirs[0];
        $this->_smarty->compile_id = $templateDirs[0];
        echo $this->_smarty->fetch($file);
    }

    /**
     * Register some php function as smarty modifiers to use in templates.
     */
    public function registerFunctions ()
    {
        $this->_smarty->register_modifier("len", "strlen");
        $this->_smarty->register_modifier("arrayLen", "sizeof");        
    }
}
?>