<?php
/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/


/**
 * SmartyView Loaders load Smarty as a Zend View and register it as a view helper. 
 * On Application startup SmartyViewLoader class created the SmartyView object and supplied the 
 * directory for compiled templates. It registers SmartyView class as a 
 * (Zend Vew through Zend_Controller_Action_Helper_ViewRenderer). This is helper class, that take 
 * care of initializing and displaying content from our view object automatically
 * 
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 * @see SmartyView
 * @see index.php (Front Controller)
 */
class SmartyViewLoader
{
    private $view = null;
    private $viewHelper = null;

    public function __construct ()
    {
        $this->init();
    }

    public function init ()
    {
        /**
         * Create the view and set the compile dir to template_c 
         */
        $this->view = new SmartyView(array('compileDir' => './templates_c'));
    }

    public function load ()
    {
        /**
         * Create a new ViewRenderer helper and assign our newly
         * created SmartyView object as the view instance
         */
        $this->viewHelper = new Zend_Controller_Action_Helper_ViewRenderer($this->view);
        $this->viewHelper->setViewSuffix('tpl');
        //Save the helper to the HelperBroker
        Zend_Controller_Action_HelperBroker::addHelper($this->viewHelper);
    }

    public function unLoad ()
    {
        Zend_Controller_Action_HelperBroker::removeHelper($this->viewHelper);
        unset($this->viewHelper);
    }

    public function getSmarty ()
    {
        return $this->view->getEngine();
    }
}
?>