<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once 'ConfigFileManagerModel.php';
require_once 'WSConstants.php';

/**
 * SpexLiveServiceClient is a simple client for connecting to an end service, sending WSRequest and retrieving 
 * it's raw XML response. It a Singleton class so only one instance of this class is created anytime.
 *
 * SpexLiveServiceClient is used to service request comming from WSGateway and send the request to the target
 * service and get the XML response from the service.
 * 
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 * @see WSGateway
 * @see WSRequest
 * @see WSResponse 
 */
class SpexLiveServiceClient
{
    /**
     * static property to hold singleton instance
     */
    private static $INSTANCE = false;
    /**
     * Reference to REST client object
     *
     * @var Zend_Rest_Client
     */
    private $_restClient = null;

    /**
     * factory method to return the singleton instance
     * @return SpexLiveServiceClient
     */
    static public function getInstance ()
    {
        if (! SpexLiveServiceClient::$INSTANCE) {
            SpexLiveServiceClient::$INSTANCE = new SpexLiveServiceClient();
        }
        return SpexLiveServiceClient::$INSTANCE;
    }

    /**
     * Performs object initializations
     * @return void
     */
    private function __construct ()
    {
    }

    /**
     * Returns a reference to the REST client, instantiating it if necessary
     *
     * @param string $WebService Location
     * @return Zend_Rest_Client
     */
    public function getRestClient ($url)
    {
        if (null === $this->_restClient) {
            /**
             * @see Zend_Rest_Client
             */
            $config = ConfigFileManagerModel::getInstance();
            require_once 'Zend/Rest/Client.php';
            $this->_restClient = new Zend_Rest_Client($url);
            // Rest Client will automatically close the connection if its idle default specified time frame.
            // Timeout and Redirects Configurtion is set in inquire.web.ini file.
            $this->_restClient->getHttpClient()->setConfig($config->getValue('timeout', 'App'));
        }
        return $this->_restClient;
    }

    /**
     * Service method send request to target web service and returns the rsponse from the service. It support
     * two types of rest client for service ws.spexlive.net.
     * 
     * <li>Zend Rest Cleint</li>
     * <li>File Get Contents</li> 
     * 
     * Any of the cleint api is used to get data from WebService.
     *
     * @param string $type
     * @param WSRequest $wsRequest
     * @return WSResponse
     */
    private function service ($type, $wsRequest)
    {
        $params = $wsRequest->getParameters();
        $client = null;
        $response = false;
        
        if ($type == WSConstants::$REST_GET) {
            $client = $this->getRestClient($wsRequest->getEndPoint());
            $client->getHttpClient()->resetParameters();
            $response = $client->restGet(WSConstants::$REST, $params);
        } else if ($type == WSConstants::$SIMPLE) {
            $config = ConfigFileManagerModel::getInstance();
            set_time_limit($config->getValue('timeout', 'SLService'));
            $requestString = $wsRequest->createRequestString();
            $response = file_get_contents($requestString);
        }
        $wsResponse = false;
        if ((false !== $response) && ('' !== $response)) {
            $wsResponse = WSFactory::createWSResponse($type, $response);
        }
        return $wsResponse;
    }

    /**
     * Call the target service through Zend_Rest_Client Api.
     *
     * @param WSRequest $wsRequest
     * @return WSResponse
     */
    public function doRest ($wsRequest)
    {
        $wsResponse = $this->service(WSConstants::$REST_GET, $wsRequest);
        return $wsResponse;
    }

    /**
     * Call the target service through file_get_contents.
     *
     * @param WSRequest $wsRequest
     * @return WSResponse
     */
    public function doSimpleGet ($wsRequest)
    {
        $wsResponse = $this->service(WSConstants::$SIMPLE, $wsRequest);
        return $wsResponse;
    }
}
?>