<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once 'WSResponse.php';
require_once 'WSRequest.php';
require_once 'WSConstants.php';

/**
 * WSFactory class is responsible for creating Webservice objects
 * 
 * @see WSRequest
 * @see WSResponse
 * @see WSException
 *
 * This class will create following object:
 *
 * <li>WSRequest</li>
 * <li>WSResponse</li>
 * <li>WSException</li>
 *
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 * @see FilterController
 * @see SearchTrail
 */
class WSFactory
{

    /**
     * Factory method for creating web service request. It will create WSRequest depending on the type
     * of request passed. 
     * 
     * Current implemention contains following types of request created by this WSRequest.
     * 
     * <li>Keyword Search Request</li>
     * <li>DisplayCategory Search Request</li>
     * <li>Product Detail Request</li>
     * <li>Feature Products Request</li>
     * <li>Product ComRequest</li>
     * <li>Ajax Request used for filters and product descriptions</li>
     * 
     * WSRequest create based on the type of request REST/SOAP and the method used to call target webservice. 
     */
    public static function createWSRequest ($type, $method, $parameters)
    {
        if ($type == WSConstants::$REST and $method == WSConstants::$SEARCH_REQUEST) {
            $request = WSRequest::createSearchRequest($parameters);
            return $request;
        } elseif ($type == WSConstants::$REST and $method == WSConstants::$COMPARISON_REQUEST) {
            $request = WSRequest::createCompareProductRequest($parameters);
            return $request;
        } elseif ($type == WSConstants::$REST and $method == WSConstants::$GET_PRODUCT_DETAIL) {
            $request = WSRequest::createProductDetailRequest($parameters);
            return $request;
        } elseif ($type == WSConstants::$REST and $method == WSConstants::$AJAX_REQUEST) {
            $request = WSRequest::createWSAjaxRequest($parameters);
            return $request;
        } elseif ($type == WSConstants::$REST and $method == WSConstants::$CATEGORY_FAMILY) {
            $request = WSRequest::createCategoryFamilyRequest($parameters);
            return $request;
        } elseif ($type == WSConstants::$REST and $method == WSConstants::$FEATURED_PRODUCTS) {
            $request = WSRequest::createSearchRequest($parameters);
            return $request;
        }
 		elseif ($type == WSConstants::$REST and $method == WSConstants::$CATEGORY_DIRECTORY) {
            $request = WSRequest::createCategoryDirectoryRequest($parameters);
            return $request;
        }
        elseif ($type == WSConstants::$REST and $method == WSConstants::$MANUFACTURER_DIRECTORY) {
            $request = WSRequest::createManufacturerDirectoryRequest($parameters);
            return $request;
        }
        return false;
    }

    /**
     * Factory method for creating WSResponse. The method created specified type of response depending on the
     * type parameter passed.
     */
    public static function createWSResponse ($type, $response)
    {
        $wsResponse = null;
        if ($type == WSConstants::$REST_GET) {
            $wsResponse = WSResponse::createZendHttpResponse($response);
        } else if ($type == WSConstants::$SIMPLE) {
            $wsResponse = WSResponse::createHttpResponse($response);
        }
        return $wsResponse;
    }

    /**
     * Factory method for creating WSException object.
     */
    public static function createWSException ()
    { // TODO Note
    // Need to shift all response creation logic here.
    }
}
?>