<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

//require_once 'WSRequest.php';
require_once 'SpexLiveServiceClient.php';
require_once 'RetryCode.php';

/**
 * This is a Gateway class to the WebService. All the web services requests will be sent through Gateway class.
 * 
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 * 
 * @see WSRequest
 * @see WSResponse
 * @see SpexLiveServiceClient
 *
 */
class WSGateway extends RetryCode
{
    /**
     * static property to hold singleton instance
     */
    private static $INSTANCE = false;

    /**
     * factory method to return the singleton instance
     * @return WSGateway
     */
    static public function getInstance ()
    {
        if (! WSGateway::$INSTANCE) {
            WSGateway::$INSTANCE = new WSGateway();
        }
        return WSGateway::$INSTANCE;
    }

    /**
     * constructor
     * private so only getInstance() method can instantiate
     * @return void
     */
    private function __construct ()
    {
    }

    /**
     * Send Request to SpexliveService through SpexLiveServiceClient. Call service
     * with retry. Retry is useful because sometimes the connection will fail for some
     * reason but will succeed when retried again.
     *
     * @param string RequestType
     * @param WSRequest $wsRequest
     * @return WSResponse
     */
    public function send ($type, $wsRequest)
    {
        /** RetryCode calling logic **/
        $response = parent::run($this, $wsRequest);
        return $response;
    }

    /**
     * This method will contain the code that would be executed using the retry logic.
     *
     * @param WSrequest $arguments
     * @return WSResponse
     */
    public function execute ($arguments)
    {
        $client = SpexLiveServiceClient::getInstance();
        $response = $client->doSimpleGet($arguments);
        return $response;
    }

    /**
     * Destroy instance of WSGateway class if not needed.
     */
    public function finalizeINSTANCE ()
    {
        unset($INSTANCE);
    }
}
?>